from datetime import datetime
import getpass
import json
import logging
import sys


from Zabbix import Zabbix, ZabbixNotAuthorized
from utils import build_parsers, parse_args
from zabbixctl import __version__ as version

log = logging.getLogger(__name__)


class ZabbixCLI(object):
    METHOD_TYPE = None
    METHOD = None
    JOBS = None
    HOSTS = None

    def __init__(self, version):
        self._args = None
        self._parser = build_parsers(version=version)
        self.JOBS = {}
        self.HOSTS = {}

    def load(self, parser_args):
        try:
            # parse the supplied args with ConfigParser.parse_args
            self._args = self._parser.parse_args(parser_args)
        except IOError as e:
            log.error("Could not open file %s: %s" %
                      (e.filename, e.strerror))
            exit(1)

        if self._args.debug:
            log.setLevel(logging.DEBUG)
            log.debug("Debug enabled")
        self.METHOD_TYPE = getattr(self._args, 'type')
        self.METHOD = getattr(self._args, 'subparser_name')

        if self.METHOD not in ['help']:
            self.HOSTS = {}
            rets = {}
            for host in self._args.hosts:
                self.HOSTS[host] = Zabbix(
                    host,
                    self._args.uri_path,
                    self._args.user,
                    self._args.noverify,
                    self._args.cacert,
                    self._args.http,
                    self._args.timeout
                )
                zapi_function = getattr(
                    getattr(getattr(self.HOSTS[host], 'zapi'), self.METHOD_TYPE), self.METHOD)

                # If the listkeys argument was supplied, we need to override
                # args.arguments to pull one resource
                if getattr(self._args, 'listkeys', False):
                    self._args.arguments = ['output=extend', 'limit=1']
                # convert the arguments into the required format for the zapi
                # object.
                args_real = parse_args(self._args.arguments)

                # Parse specific cli arguments and update args_real
                args_to_parse = ['search', 'filter']
                for key in args_to_parse:
                    if getattr(self._args, key, None) is not None:
                        args_real[key] = parse_args(getattr(self._args, key))

                self.JOBS[host] = (zapi_function, args_real)

    def auth(self):
        for host, zbx in self.HOSTS.items():
            count = 0
            while count < 3 and zbx.zapi.auth == '':
                try:
                    zbx.auth(self._args.user, getpass.getpass())
                    # if successful, break out of the loop
                    break
                except ZabbixNotAuthorized:
                    pass
                count += 1
            if zbx.zapi.auth == '':
                log.exception(ZabbixNotAuthorized(
                    'Invalid username or password for {0}'.format(host)))

    def execute(self):
        if self.METHOD not in ['help']:
            final = []
            for job, data in self.JOBS.items():
                zapi_function = data[0]
                arguments = data[1]
                if type(arguments) == str or type(arguments) == int:
                    ret = zapi_function(str(arguments))
                elif type(arguments) == list:
                    ret = zapi_function(*arguments)
                else:
                    ret = zapi_function(**arguments)

                if type(ret) == list:
                    final += ret
                elif type(ret) == dict:
                    final = ret
                else:
                    final = eval(ret)

            # if method type is alert, sort based on 'clock' key
            if any(self.METHOD_TYPE in s for s in ['alert']):
                final = sorted(final, key=lambda k: k['clock'])

            if self.METHOD not in ['create', 'delete', 'update', 'export']:
                # Check if one of the following keys exist
                list_of_keys = ['clock', 'lastchange']
                matched_key = None
                for key in list_of_keys:
                    try:
                        if final[0].get(key, None):
                            matched_key = key
                    except IndexError:
                        pass
                # If a key exists, then sort on that key and update the unix_timestamp
                # to readable format
                if matched_key:
                    final = sorted(final, key=lambda k: k[matched_key])
                    for item in final:
                        item[matched_key] = str(
                            datetime.fromtimestamp(float(item[matched_key]))
                        )
            # if the "listkeys" argument was supplied, we should return the
            # keys of the first resource in the list.
            if getattr(self._args, 'listkeys', False):
                sys.stdout.write(json.dumps(final[0].keys(), indent=2))
            else:
                sys.stdout.write(json.dumps(final, indent=2))
        else:
            log.info(
                'https://www.zabbix.com/documentation/2.2/manual/api/reference/{0}'.format(self.METHOD_TYPE)
            )


def main(args=None):
    handler = logging.StreamHandler(sys.stdout)
    log.addHandler(handler)
    log.setLevel(logging.INFO)
    logging.captureWarnings(True)

    cli = ZabbixCLI(version=version)
    cli.load(sys.argv[1:])
    cli.auth()
    cli.execute()



import getpass
import argparse
import json
import logging
import sys


class Cache:

    def __init__(self, cachefile):
        self.cachefile = cachefile

    def get(self, slug):
        """
        Get token from cache
        :param slug: unique key to store in cache.  Should be 'host-user'.
        """
        try:
            token_data = json.load(open(self.cachefile, 'rb'))
            token = token_data.get(slug, None)
        except IOError:
            token = None
        return token

    def write(self, slug, token):
        try:
            token_data = json.load(open(self.cachefile, 'rb'))
        except IOError:
            token_data = {}
        token_data[slug] = token
        json.dump(token_data, open(self.cachefile, 'wb'))


def build_parsers(version):
    """Build parsers for cli"""
    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter,
                                     description='Zabbix CLI')
    parser.add_argument('-v', '--version', action='version',
                        version='%(prog)s {0}'.format(version))
    parser.add_argument('-d', '--debug',
                        dest='debug',
                        help='increase output verbosity', action='store_true')
    parser.add_argument('-i', '--http',
                        help='Use http instead of https', action='store_true')
    parser.add_argument('-V', '--noverify',
                        dest='noverify',
                        help='Do not verify the SSL', action='store_true')
    parser.add_argument('-c', '--cacert',
                        dest='cacert',
                        help='Path to the SSL CA Certificate'
                        'example: /etc/pki/tls/certs/ca-bundle.crt')
    parser.add_argument('-t', '--timeout',
                        dest='timeout',
                        default=30,
                        type=float,
                        help='Zabbix API read timeout in seconds')
    parser.add_argument('-H', '--hosts',
                        dest='hosts',
                        required=True,
                        help='Zabbix API host(s).'
                        'example: zabbixhost.example.com', action='append')
    parser.add_argument('-p', '--uri-path',
                        dest='uri_path',
                        default='zabbix',
                        help='URI path to zabbix api. default: zabbix')
    parser.add_argument('-U', '--user',
                        default=getpass.getuser(),
                        help='Zabbix API user')

    subparsers = parser.add_subparsers(dest='subparser_name',)

    get_parser = subparsers.add_parser('get', help='Zabbix API Method for get')
    get_parser.add_argument('type',
                            help='Zabbix API get method (host.get,'
                            'hostgroups.get,usergroups.get)')
    get_parser.add_argument('-a', '--arguments',
                            dest='arguments',
                            default=['output=extend', ],
                            help='RPC params', action='append')
    get_parser.add_argument('-k', '--listkeys',
                            dest='listkeys',
                            default=False,
                            help='Returns a list of keys for the '
                            'given resource type', action='store_true')
    get_parser.add_argument('-f', '--filter',
                            dest='filter',
                            help='Takes "key=value" args that are sent to the'
                            ' zabbix api in the filter parameter', action='append')
    get_parser.add_argument('-s', '--search',
                            dest='search',
                            help='Takes "key=value" args that are sent to the'
                            ' zabbix api in the search parameter', action='append')

    export_parser = subparsers.add_parser(
        'export', help='Zabbix API Method for export')
    export_parser.add_argument('type',
                               help='Zabbix API export method (host.export,'
                               'hostgroups.export,usergroups.export)')
    export_parser.add_argument('-a', '--arguments',
                               dest='arguments',
                               help='RPC params', action='append')

    update_parser = subparsers.add_parser(
        'update', help='Zabbix API Method for update')
    update_parser.add_argument('type',
                               help='Zabbix API update method (host.update,'
                               'hostgroups.update,usergroups.update)')
    update_parser.add_argument('-a', '--arguments',
                               dest='arguments',
                               help='RPC params', action='append')

    create_parser = subparsers.add_parser(
        'create', help='Zabbix API Method for create')
    create_parser.add_argument('type',
                               help='Zabbix API get method (host.create,'
                               'hostgroups.create,usergroups.create)')
    create_parser.add_argument('-a', '--arguments',
                               dest='arguments',
                               help='RPC params', action='append')

    delete_parser = subparsers.add_parser(
        'delete', help='Zabbix API Method for delete')
    delete_parser.add_argument('type',
                               help='Zabbix API get method (host.delete,'
                               'hostgroups.delete,usergroups.delete)')
    delete_parser.add_argument('-a', '--arguments',
                               dest='arguments',
                               help='RPC params', action='append')

    help_parser = subparsers.add_parser(
        'help', help='Display link for Zabbix wiki')
    help_parser.add_argument('type',
                             help='Displays link for Zabbix Wiki')

    return parser


def parse_args(args):
    """
    Takes the given args, parses them and return the arguments object
    :param args:
    :return:
    """
    arguments = {}
    if args is not None:
        for argument in args:
            if '=' in argument:
                tmp = [a for a in argument.split('=', 1)]
                try:
                    value = eval(tmp[1])  # todo: this seems dangerous
                except (NameError, SyntaxError):
                    value = tmp[1]
                arguments[tmp[0]] = value
            else:
                arguments = eval(argument)
    return arguments



"""Contains all the classes and functions associated with the Zabbix object
type"""
# Standard Lib
import getpass
import logging
from utils import Cache
from urlparse import urlunparse, urljoin

# Packages
from pyzabbix import ZabbixAPI, ZabbixAPIException
from requests.exceptions import HTTPError, ConnectionError

log = logging.getLogger(__name__)


class ZabbixError(Exception):
    pass


class ZabbixNotAuthorized(ZabbixError):
    pass


class Zabbix(object):

    def __init__(self, host, uri_path, user, noverify=False, cacert=None, http=False, timeout=30):
        """
        Initializes a Zabbix instance
        :param host: hostname to connect to (ex. zabbix.yourdomain.net)
        :param user: username to connect with (ex. Admin)
        :param uri_path: uri path to zabbix api (ex. zabbix)
        :param noverify: turns off verification
        :param cacert: the certificate authority to use
        :param http: flag to use http over https
        :param timeout: API timeout parameter
        :return: Zabbix instance
        """

        self.cache = Cache('/tmp/zabbix.cache')
        self.host = host
        self.cache_slug = '{0}-{1}'.format(host, user)

        zabbix_url = urlunparse([
            'http' if http else 'https',
            host.strip('/'),
            uri_path,
            '', '', ''
        ]
        )
        log.debug("Creating instance of Zabbic with url: %s", zabbix_url)

        self.zapi = ZabbixAPI(zabbix_url)

        if cacert is not None:
            log.debug('Setting zapi.session.verify to {0}'
                      ''.format(cacert))
            self.zapi.session.verify = cacert

        if noverify:
            log.debug('Setting zapi.session.verify to False')
            self.zapi.session.verify = False

        self.zapi.timeout = timeout
        self.fetch_zabbix_api_version()  # Check the api

        token = self.cache.get(self.cache_slug)
        if token:
            log.debug('Found token for {0}'.format(host))
            self.zapi.auth = token
            # Let's test the token by grabbing the api version
            try:
                self.fetch_zabbix_api_version()
            except ZabbixNotAuthorized:
                self.zapi.auth = ''

    def fetch_zabbix_api_version(self):
        """
        reaches out to the zapi api info to parse the string
        :return: Version string or False
        """
        try:
            return self.zapi.apiinfo.version()

        except (HTTPError, ConnectionError, ZabbixAPIException) as e:
            # todo: cant we check by the error, not its string?
            if 'Not authorized' in str(e):
                log.debug('Token not authorized for {0}'.format(self.host))
                raise ZabbixNotAuthorized
            raise ZabbixError(e)

    def auth(self, username, password):
        """
        Performs the loggin function of the api with the supplied credentials
        :param username: username
        :param password: password
        :return: True is valid, False otherwise
        """
        try:
            self.zapi.login(username, password)
            self.cache.write(self.cache_slug, self.zapi.auth)
        except ZabbixAPIException as e:
            raise ZabbixNotAuthorized('Username or password invalid')
        return True


if __name__ == '__main__':
    Z_obj = Zabbix('zabbix.yourdomain.net')
    username = getpass.getuser()
    password = getpass.getpass()
    Z_obj.auth(username, password)
    import pdb
    pdb.set_trace()



from setuptools import setup
from sys import path

from zabbixctl import __version__

path.insert(0, '.')

NAME = "zabbixctl"

if __name__ == "__main__":

    with open('requirements.txt') as f:
        requirements = f.read().splitlines()

    setup(
        name=NAME,
        version=__version__,
        author="Tony Rogers",
        author_email="tony.rogers@rackspace.com",
        url="https://github.com/teriyakichild/zabbixctl",
        license='ASLv2',
        packages=[NAME],
        package_dir={NAME: NAME},
        description="zabbixctl - Utility that connects to Zabbix API",

        install_requires=requirements,

        entry_points={
            'console_scripts': ['zabbixctl = zabbixctl.cli:main'],
        }
    )
